@@ -98,6 +98,10 @@ public:
                              time_t start,
                              time_t end,
                              kodi::addon::PVREPGTagsResultSet& results) override;
+  PVR_ERROR GetEPGForChannelNew(int channelUid,
+                                time_t start,
+                                time_t end,
+                                kodi::addon::PVREPGTagsResultSet& results);
   PVR_ERROR IsEPGTagRecordable(const kodi::addon::PVREPGTag& tag, bool& isRecordable) override;
   PVR_ERROR IsEPGTagPlayable(const kodi::addon::PVREPGTag& tag, bool& isPlayable) override;
   PVR_ERROR GetEPGTagStreamProperties(
@@ -105,6 +109,7 @@ public:
       std::vector<kodi::addon::PVRStreamProperty>& properties) override;
 
   PVR_ERROR GetRecordingsAmount(bool deleted, int& amount) override;
+  kodi::addon::PVRRecording ParseRecordingEntry(const rapidjson::Value& recordingEntry);
   PVR_ERROR GetRecordings(bool deleted, kodi::addon::PVRRecordingsResultSet& results) override;
   PVR_ERROR DeleteRecording(const kodi::addon::PVRRecording& recording) override;
   PVR_ERROR GetRecordingStreamProperties(
@@ -118,6 +123,10 @@ public:
   PVR_ERROR AddTimer(const kodi::addon::PVRTimer& timer) override;
   PVR_ERROR GetDriveSpace(uint64_t& total, uint64_t& used) override;
   PVR_ERROR OnSystemWake() override;
+  PVR_ERROR GetRecordingLastPlayedPosition(const kodi::addon::PVRRecording& recording,
+                                           int& position) override;
+  PVR_ERROR SetRecordingLastPlayedPosition(const kodi::addon::PVRRecording& recording,
+                                           int lastplayedposition) override;
 
 private:
   bool m_isConnected = false;
@@ -126,7 +135,8 @@ private:
   std::thread m_loginThread;
   void LoginThread();
   time_t m_nextLoginAttempt = 0;
-  WAIPU_CHANNEL_IMPORT_FILTER m_channel_filter = WAIPU_CHANNEL_IMPORT_FILTER::CHANNEL_FILTER_ALL_VISIBLE;
+  WAIPU_CHANNEL_IMPORT_FILTER m_channel_filter =
+      WAIPU_CHANNEL_IMPORT_FILTER::CHANNEL_FILTER_ALL_VISIBLE;
 
   struct WaipuChannel
   {
@@ -150,7 +160,7 @@ private:
   std::string m_userhandle = "";
   std::string m_protocol;
   std::string m_device_id;
-  WAIPU_PROVIDER m_provider;
+  WAIPU_PROVIDER m_provider = WAIPU_PROVIDER_WAIPU;
 
   std::vector<WaipuChannel> m_channels;
   std::vector<WaipuChannelGroup> m_channelGroups;
@@ -173,11 +183,12 @@ private:
   WAIPU_LOGIN_STATUS m_login_status = WAIPU_LOGIN_STATUS::UNKNOWN;
   HLSAllowlist m_hls_allowlist;
   Categories m_categories;
+  time_t m_lastUpdate = 0;
 
   void ReadSettings();
   bool ParseAccessToken();
 
-  void AddTimerType(std::vector<kodi::addon::PVRTimerType>& types, int idx, int attributes);
+  void AddTimerType(std::vector<kodi::addon::PVRTimerType>& types, int id, int attributes);
 
   std::string GetChannelStreamURL(int uniqueId,
                                   const std::string& protocol,
@@ -185,7 +196,7 @@ private:
   std::string GetRecordingURL(const kodi::addon::PVRRecording& recording,
                               const std::string& protocol);
   std::string GetEPGTagURL(const kodi::addon::PVREPGTag& tag, const std::string& protocol);
-  std::string GetLicense();
+  const std::string& GetLicense();
   const std::map<std::string, std::string> GetOAuthDeviceCode(const std::string& tenant);
   const std::map<std::string, std::string> CheckOAuthState(const std::string& device_code);
   void SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty>& properties,
